VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "ThisWorkbook"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = True
' last update: 2012-08-14
' HATANO Hirokazu
'

' Also See: http://d.hatena.ne.jp/language_and_engineering/20090731/p1
' 1. 手動リロード関数(reloadModule)を追加
' 2. ライブラリリストについて、 ../ を使った相対指定(例: ../common/hoge.bas)への対応を追加
' 3. ライブラリリストについて、パス区切りなしの指定(例: hoge.bas)への対応を追加
' 4. ライブラリリストで使うパス区切り文字について、Windowns(\)/Mac(:)/UNIX(/) のどれを
'     使っても良いようにした。
' 5. 定数 ENABLE_WORKBOOK_OPEN の値でファイルオープン時の挙動をON/OFFできるようにした
' 6. Mac版Office でも動作するようにした。
' 7. このExcelファイル自体のUNITテスト用関数を追加してみた。(試験的)


Option Explicit

'----------------------------- Consts ---------------

'ライブラリリストの設定 (設置フォルダはワークブックと同じディレクトリ)
Const FILENAME_LIBLIST As String = "libdef.txt" 'ライブラリリストのファイル名
Const FILENAME_EXPORT As String = "ThisWorkbook-sjis.cls" 'エクスポート clsファイル名

'ワークブック オープン時に実行する(True) / しない(False)
Const ENABLE_WORKBOOK_OPEN As Boolean = True
'Const ENABLE_WORKBOOK_OPEN As Boolean = False



'----------------------------- Workbook_open() ---------------

'ワークブック オープン時に実行
Private Sub Workbook_Open()
  If ENABLE_WORKBOOK_OPEN = False Then
    Exit Sub
  End If
  
  Call reloadModule
 End Sub



'----------------------------- public Subs/Functions ---------------

Public Sub reloadModule()
  '手動リロード用 Public関数
  
  Dim msgError As String
  msgError = loadModule("." & Application.PathSeparator & FILENAME_LIBLIST)
  
  If Len(msgError) > 0 Then
    MsgBox msgError
  End If
End Sub


Public Sub exportThisWorkbook()
  'ThisWorkbook 手動export用 Public関数
  Call exportModule("ThisWorkbook", FILENAME_EXPORT)
End Sub


'----------------------------- main Subs/Functions ---------------

Private Function loadModule(ByVal pathConf As String) As String
  'Main: モジュールリストファイルに書いてある外部ライブラリを読み込む。

  '1. 全モジュールを削除
  Dim isClear As Boolean
  isClear = clearModules
  
  If isClear = False Then
    loadModule = "Error: 標準モジュールの全削除に失敗しました。"
    Exit Function
  End If
  
  
  '2. モジュールリストファイルの存在確認
  ' 2.1. モジュールリストファイルの絶対パスを取得
  pathConf = absPath(pathConf)
  
  ' 2.2. 存在チェック
  Dim isExistList As Boolean
  isExistList = checkExistFile(pathConf)
  
  If isExistList = False Then
    loadModule = "Error: ライブラリリスト" & pathConf & "が存在しません。"
    Exit Function
  End If


  '3. モジュールリストファイルの読み込み&配列化
  Dim arrayModules As Variant
  arrayModules = list2array(pathConf)
  
  If UBound(arrayModules) = 0 Then
    loadModule = "Error: ライブラリリストに有効なモジュールの記述が存在しません。"
    Exit Function
  End If

  
  '4. 各モジュールファイル読み込み
  Dim i As Integer
  Dim msgError As String
  msgError = ""
  
  ' 配列は0始まり。(最大値: 配列個数-1)
  For i = 0 To UBound(arrayModules) - 1
    Dim pathModule As String
    pathModule = arrayModules(i)
    
    '4.1. モジュールリストファイルの存在確認
    ' 4.1.1. モジュールリストファイルの絶対パスを取得
    pathModule = absPath(pathModule)
  
    ' 4.1.2. 存在チェック
    Dim isExistModule As Boolean
    isExistModule = checkExistFile(pathModule)
  
    '4.2. モジュール読み込み
    If isExistModule = True Then
      ThisWorkbook.VBProject.VBComponents.Import pathModule
    Else
      msgError = msgError & pathModule & " は存在しません。" & vbCrLf
    End If
  Next i
  loadModule = msgError

End Function



'----------------------------- Functions / Subs ---------------



Private Sub exportModule(ByVal nameModule As String, ByVal nameFile As String)

  Dim component As Object
  For Each component In ThisWorkbook.VBProject.VBComponents
    
    If component.Name = nameModule Then
      component.Export ThisWorkbook.path & Application.PathSeparator & nameFile
      MsgBox nameModule & " を " & ThisWorkbook.path & Application.PathSeparator & nameFile & " として保存しました。"
    End If
    
  Next component

End Sub




'----------------------------- common Functions / Subs ---------------
Private Function clearModules() As Boolean
  '標準モジュール初期化(全削除)
  
  Dim component As Object
  For Each component In ThisWorkbook.VBProject.VBComponents
      
    '標準モジュール(Type=1)を全て削除
    If component.Type = 1 Then
      ThisWorkbook.VBProject.VBComponents.Remove component
    End If
    
  Next component
  
  '標準モジュールの数が0であればOK
  Dim count As Long
  count = countModules()
        
  If count = 0 Then
    clearModules = True
  Else
    clearModules = False
  End If
End Function





Private Function countModules()
  '存在する標準モジュールの数を数える
  
  Dim i As Long
  Dim count As Long
  count = 0
  
  With ThisWorkbook.VBProject
    For i = 1 To .VBComponents.count
      If .VBComponents(i).Type = 1 Then
        count = count + 1
      End If
    Next i
  End With

  countModules = count
End Function



Private Function absPath(ByVal pathFile As String) As String
  ' ファイルパスを絶対パスに変換
  
  'replace Win "\" / Mac ":" / Unix "/"
  pathFile = Replace(pathFile, "\", Application.PathSeparator)
  pathFile = Replace(pathFile, ":", Application.PathSeparator)
  pathFile = Replace(pathFile, "/", Application.PathSeparator)
  
  Dim strHead As String
  strHead = Left(pathFile, 2)
  
  Select Case strHead
    Case "..":
      pathFile = ThisWorkbook.path & Application.PathSeparator & pathFile
      
    Case "." & Application.PathSeparator:
      pathFile = ThisWorkbook.path & Mid(pathFile, 2, Len(pathFile) - 1)
  
    Case Else:
      
      Select Case True
      
        Case Left(pathFile, 2) Like "[0-9][A-z]" Or Left(pathFile, 2) Like "[0-9][0-9]" _
          Or Left(pathFile, 2) Like "[A-z][0-9]" Or Left(pathFile, 2) Like "[A-z][A-z]":
        pathFile = ThisWorkbook.path & Application.PathSeparator & pathFile

        Case Else:
          'MsgBox strHead
      End Select
    
  End Select
    
  absPath = pathFile
End Function



Private Function checkExistFile(ByVal pathFile As String) As Boolean

  On Error GoTo Err_dir
  If Dir(pathFile) = "" Then
    checkExistFile = False
  Else
    checkExistFile = True
  End If

  Exit Function

Err_dir:
  checkExistFile = False

End Function



'リストファイルを配列で返す(行頭が'(コメント)の行 & 空行は無視する)
Private Function list2array(ByVal pathFile As String) As Variant
    
  '1. リストファイルの読み取り
  Dim fp As Integer
  fp = FreeFile
  Open pathFile For Input As #fp
  
  '2. リストの配列化
  Dim arrayOutput() As String
  Dim countLine As Integer
  countLine = 0
  ReDim Preserve arrayOutput(countLine) ' 配列0で返す場合があるため
  
  Do Until EOF(fp)
    'ライブラリリストを1行ずつ処理
    Dim strLine As String
    Line Input #fp, strLine

    '行頭が '(コメント) ではない & 空行ではない場合
    If Not Left(strLine, 1) = "'" And Len(strLine) > 0 Then
      
      '配列への追加
      countLine = countLine + 1
      ReDim Preserve arrayOutput(countLine)
      arrayOutput(countLine - 1) = strLine
    End If
  Loop

  '3. リストファイルを閉じる
  Close #fp
  
  '4. 戻り値を配列で返す
  list2array = arrayOutput
End Function



'----------------------------- Tests ---------------

Public Sub test_ThisWorkbook()
  Dim strError As String
  
  'Unit tests
  strError = strError & test_absPath
  strError = strError & test_clearModules
  strError = strError & test_checkExistFile()
  strError = strError & test_list2array
  '
  'strError = strError & test_loadModule
  
  If Len(strError) > 0 Then
    MsgBox strError
  Else
    MsgBox "OK! (No Error)"
  End If
End Sub



Private Sub test_loadModule()
  Dim strError As String
  strError = ""

'  Dim result As Boolean
'  result = loadModule()

test_loadModule = strError
End Sub



Private Function test_absPath()
  Dim strError As String
  strError = ""

  Dim pathFile As String
  Dim result As String
  
  Dim sp As String
  sp = Application.PathSeparator
  
  '
  ' "../test.bas -> /path/to/../test.bas
  '
  
  'case 1
  result = absPath("..\test.bas")
  If Not result = ThisWorkbook.path & sp & ".." & sp & "test.bas" Then
    strError = strError & "[absPath case1] Error: get unexpected path & file" & vbCrLf
  End If
  
  'case 2
  result = absPath("..:test.bas")
  If Not result = ThisWorkbook.path & sp & ".." & sp & "test.bas" Then
    strError = strError & "[absPath case2] Error: get unexpected path & file" & vbCrLf
  End If
  
  'case 3
  result = absPath("../test.bas")
  If Not result = ThisWorkbook.path & sp & ".." & sp & "test.bas" Then
    strError = strError & "[absPath case3] Error: get unexpected path & file" & vbCrLf
  End If
      
      
  '
  ' "./test.bas -> /path/to/test.bas
  '
  
  'case 4
  result = absPath(".\test.bas")
  If Not result = ThisWorkbook.path & sp & "test.bas" Then
    strError = strError & "[absPath case4] Error: get unexpected path & file." & vbCrLf
  End If

  'case 5
  result = absPath(".:test.bas")
  If Not result = ThisWorkbook.path & sp & "test.bas" Then
    strError = strError & "[absPath case5] Error: get unexpected path & file." & vbCrLf
  End If

  'case 6
  result = absPath("./test.bas")
  If Not result = ThisWorkbook.path & sp & "test.bas" Then
    strError = strError & "[absPath case6] Error: get unexpected path & file." & vbCrLf
  End If


  
  '
  ' (Windows only) "c:\test.bas -> (not modify)
  '
  
  'case 7
  result = absPath("c:\test.bas")
  If Not result = "c:" & sp & "test.bas" Then
    strError = strError & "[absPath case7] Error: get unexpected path & file." & result & vbCrLf
  End If


  '
  ' (Windows only) "\\fileserver\test.bas -> (not modify)
  '

  'case 8
  result = absPath("\\filserver\test.bas")
  If Not result = sp & sp & "filserver" & sp & "test.bas" Then
    strError = strError & "[absPath case8] Error: get unexpected path & file." & result & vbCrLf
  End If


  '
  ' (Except Windows) "/test.bas -> (not modify) (Mac/Unix style)
  '
  
  'case 9
  result = absPath(":test.bas")
  If Not result = sp & "test.bas" Then
    strError = strError & "[absPath case9] Error: get unexpected path & file." & vbCrLf
  End If

  'case 10
  result = absPath("/test.bas")
  If Not result = sp & "test.bas" Then
    strError = strError & "[absPath case10] Error: get unexpected path & file." & vbCrLf
  End If


  '
  ' "test.bas -> /path/to/test.bas
  '
  
  'case 11 [A-z][A-z]
  result = absPath("test.bas")
  If Not result = ThisWorkbook.path & sp & "test.bas" Then
    strError = strError & "[absPath case11] Error: get unexpected path & file." & result & vbCrLf
  End If

  'case 12 [0-9][0-9]
  result = absPath("00st.bas")
  If Not result = ThisWorkbook.path & sp & "00st.bas" Then
    strError = strError & "[absPath case12] Error: get unexpected path & file." & result & vbCrLf
  End If

  'case 13 [A-z][0-9]
  result = absPath("t0st.bas")
  If Not result = ThisWorkbook.path & sp & "t0st.bas" Then
    strError = strError & "[absPath case13] Error: get unexpected path & file." & result & vbCrLf
  End If

  'case 14 [0-9][A-z]
  result = absPath("0est.bas")
  If Not result = ThisWorkbook.path & sp & "0est.bas" Then
    strError = strError & "[absPath case14] Error: get unexpected path & file." & result & vbCrLf
  End If

  test_absPath = strError
End Function



Private Function test_checkExistFile()
  Dim strError As String
  strError = ""
  
  Dim pathFile As String
  Dim isExistFile As Boolean
  
  'Case 1. Exist file
  pathFile = "." & Application.PathSeparator & FILENAME_LIBLIST
  pathFile = absPath(pathFile)
  isExistFile = checkExistFile(pathFile)
  If isExistFile = False Then
    strError = strError & "[checkExistFile case1] Error: fail to check exist file." & vbCrLf
  End If

  'Case 2. not Exist file
  pathFile = "." & Application.PathSeparator & "non-exist.txt"
  pathFile = absPath(pathFile)
  isExistFile = checkExistFile(pathFile)
  If isExistFile = True Then
    strError = strError & "[checkExistFile case2] Error: fail to fetch FALSE." & vbCrLf
  End If

  'Case 3. not Exist directory
  pathFile = Application.PathSeparator & "non-exist.txt"
  isExistFile = checkExistFile(pathFile)
  If isExistFile = True Then
    strError = strError & "[checkExistFile case2] Error: fail to fetch FALSE." & vbCrLf
  End If

End Function



Private Function test_list2array()
  Dim strError As String
  strError = ""
  
  Dim pathFile As String
  Dim arrayOutput As Variant
  
  'Case 1. 混合
  pathFile = "." & Application.PathSeparator & "test" & Application.PathSeparator & "libdef-mixed.txt"
  pathFile = absPath(pathFile)
  arrayOutput = list2array(pathFile)
  If UBound(arrayOutput) = 0 Then
    strError = strError & "[list2array case1] Error: fail to get list" & vbCrLf
  End If

  'Case 2. コメント(')行のみ  (./test/libdef-allcomment.txt)
  pathFile = "." & Application.PathSeparator & "test" & Application.PathSeparator & "libdef-allcomment.txt"
  pathFile = absPath(pathFile)
  arrayOutput = list2array(pathFile)
  If UBound(arrayOutput) > 0 Then
    strError = strError & "[list2array case2] Error: get some unexpected list." & vbCrLf
  End If
  
  'Case 3. 空行のみ (./test/libdef-allcomment.txt)
  pathFile = "." & Application.PathSeparator & "test" & Application.PathSeparator & "libdef-allnull.txt"
  pathFile = absPath(pathFile)
  arrayOutput = list2array(pathFile)
  If UBound(arrayOutput) > 0 Then
    strError = strError & "[list2array case3] Error: get some unexpected list." & vbCrLf
  End If
  
End Function



Private Function test_clearModules()
  Dim strError As String
  strError = ""
  
  '標準モジュールを全て削除
  Call clearModules
  
  '標準モジュールの数が0であればOK
  Dim count As Long
  count = countModules()
        
  If count > 0 Then
    'Error: 標準モジュールが存在する場合
    strError = strError & "[clearModule] Fail to dalete all standard modules." & vbCrLf
  End If
  
  test_clearModules = strError
End Function




